/*
********************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   (C) Copyright Taligent, Inc.,  1997                                        *
*   (C) Copyright International Business Machines Corporation,  1997-1998      *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
*
* File UCHAR.CPP
*
* Modification History:
*
*   Date        Name        Description
*   04/02/97    aliu        Creation.
*
*   4/15/99     Madhu        Updated all the function definitions for C Implementation
*   5/20/99     Madhu		Added the function u_getVersion()
*   8/19/1999   srl            Upgraded scripts to Unicode3.0 
********************************************************************************************
*/
#include "utypes.h"
#include "ucmp16.h"
#include "ucmp8.h"
#include "umutex.h"
#include "uchar.h"
#include "cmemory.h"
#include <string.h>

#include <stdio.h>

struct UCharDigitPair{
    uint16_t fUnicode;
    int8_t     fValue;
};
typedef struct UCharDigitPair UCharDigitPair;

struct BlockScriptMap {
    UChar        fFirstCode;
    UChar        fLastCode;
};
typedef struct BlockScriptMap BlockScriptMap;

static  bool_t  tablesCreated=FALSE;
static  bool_t  ulTablesCreated=FALSE;
static  bool_t  dirTablesCreated=FALSE;
static  void    createTables(void);
static  void    createUlTables(void);
static  void    createDirTables(void);

char* _ucdVersion=NULL;

const   UChar   MIN_VALUE = 0x0000;
const   UChar   MAX_VALUE = 0xffff;
static CompactByteArray *tables;
static CompactShortArray *ulTables; 
static CompactByteArray *dirTables;


static const uint16_t indicies[] = {

    (uint16_t)0, (uint16_t)127, (uint16_t)255, (uint16_t)382, 
    (uint16_t)502, (uint16_t)584, (uint16_t)712, 
    (uint16_t)839, (uint16_t)967, (uint16_t)1093, 
    (uint16_t)1215, (uint16_t)1335, (uint16_t)1452, 
    (uint16_t)1565, (uint16_t)1693, (uint16_t)1821, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1997, 
    (uint16_t)2124, (uint16_t)2250, (uint16_t)2377, 
    (uint16_t)2504, (uint16_t)2630, (uint16_t)2757, 
    (uint16_t)2883, (uint16_t)3009, (uint16_t)3135, 
    (uint16_t)3262, (uint16_t)3389, (uint16_t)3517, 
    (uint16_t)3645, (uint16_t)3773, (uint16_t)3869, 
    (uint16_t)3997, (uint16_t)4052, (uint16_t)4180, 
    (uint16_t)4301, (uint16_t)4414, (uint16_t)4539, 
    (uint16_t)4666, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4686, (uint16_t)4814, 
    (uint16_t)1870, (uint16_t)4942, (uint16_t)5070, 
    (uint16_t)5198, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)5326, 
    (uint16_t)5432, (uint16_t)5560, (uint16_t)5688, 
    (uint16_t)5816, (uint16_t)5944, (uint16_t)6072, 
    (uint16_t)6196, (uint16_t)6324, (uint16_t)6338, 
    (uint16_t)6466, (uint16_t)6591, (uint16_t)6719, 
    (uint16_t)6819, (uint16_t)6947, (uint16_t)7053, 
    (uint16_t)7181, (uint16_t)1870, (uint16_t)7308, 
    (uint16_t)7426, (uint16_t)6947, (uint16_t)6947, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)7554, 
    (uint16_t)6947, (uint16_t)7584, (uint16_t)7712, 
    (uint16_t)7819, (uint16_t)7946, (uint16_t)8059, 
    (uint16_t)8187, (uint16_t)8315, (uint16_t)8443, 
    (uint16_t)8566, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)8694, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)8710, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)8838, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)1870, (uint16_t)1870, 
    (uint16_t)1870, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)4667, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)8966, (uint16_t)9094, 
    (uint16_t)9094, (uint16_t)9094, (uint16_t)9094, 
    (uint16_t)9094, (uint16_t)9094, (uint16_t)9094, 
    (uint16_t)9094, (uint16_t)9094, (uint16_t)9094, 
    (uint16_t)9094, (uint16_t)9094, (uint16_t)9094, 
    (uint16_t)9094, (uint16_t)9094, (uint16_t)9094, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)9222, 
    (uint16_t)9222, (uint16_t)9222, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)8702, (uint16_t)1870, 
    (uint16_t)9350, (uint16_t)9428, (uint16_t)4667, 
    (uint16_t)4667, (uint16_t)9511, (uint16_t)9623, 
    (uint16_t)9747, (uint16_t)9865, (uint16_t)9993, 
    (uint16_t)10106 };

static const int8_t values[] = {
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)12, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)25, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)20, 
    (int8_t)21, (int8_t)23, (int8_t)24, (int8_t)23, (int8_t)19, 
    (int8_t)23, (int8_t)23, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)23, (int8_t)23, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)23, (int8_t)23, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)20, (int8_t)23, (int8_t)21, (int8_t)26, (int8_t)22, 
    (int8_t)26, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)20, (int8_t)24, (int8_t)21, 
    (int8_t)24, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)15, 
    (int8_t)15, (int8_t)15, (int8_t)15, (int8_t)12, (int8_t)23, 
    (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)27, 
    (int8_t)27, (int8_t)26, (int8_t)27, (int8_t)2, (int8_t)28, 
    (int8_t)24, (int8_t)19, (int8_t)27, (int8_t)26, (int8_t)27, 
    (int8_t)24, (int8_t)11, (int8_t)11, (int8_t)26, (int8_t)2, 
    (int8_t)27, (int8_t)23, (int8_t)26, (int8_t)11, (int8_t)2, 
    (int8_t)29, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)23, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)24, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)24, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)5, (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)1, 
    (int8_t)3, (int8_t)2, (int8_t)1, (int8_t)3, (int8_t)2, 
    (int8_t)1, (int8_t)3, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)1, 
    (int8_t)3, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)0, (int8_t)0, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)26, (int8_t)26, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)4, (int8_t)4, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)4, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)26, (int8_t)26, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)4, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)23, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)26, (int8_t)26, (int8_t)1, 
    (int8_t)23, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)0, 
    (int8_t)1, (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)0, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)27, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)7, (int8_t)7, (int8_t)0, (int8_t)0, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, 
    (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)0, (int8_t)0, (int8_t)4, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)0, (int8_t)23, (int8_t)19, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)23, (int8_t)6, (int8_t)23, (int8_t)6, 
    (int8_t)6, (int8_t)23, (int8_t)6, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)23, 
    (int8_t)23, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)23, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)23, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)23, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)4, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)23, (int8_t)5, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)7, (int8_t)7, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)4, (int8_t)4, (int8_t)6, (int8_t)6, (int8_t)27, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)0, (int8_t)16, (int8_t)5, (int8_t)6, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)8, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)5, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)8, 
    (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)6, 
    (int8_t)6, (int8_t)23, (int8_t)23, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)23, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)8, (int8_t)8, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)0, 
    (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)5, (int8_t)5, (int8_t)25, (int8_t)25, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)6, (int8_t)6, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)8, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)5, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)8, 
    (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)8, (int8_t)8, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)5, 
    (int8_t)8, (int8_t)6, (int8_t)8, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)8, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, 
    (int8_t)8, (int8_t)8, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, 
    (int8_t)6, (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)6, (int8_t)8, (int8_t)8, 
    (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)8, (int8_t)8, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)8, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, 
    (int8_t)6, (int8_t)0, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)8, (int8_t)8, 
    (int8_t)23, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)6, (int8_t)5, (int8_t)5, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)25, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)4, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)23, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)23, (int8_t)23, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)6, (int8_t)5, (int8_t)5, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)4, (int8_t)0, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)6, (int8_t)6, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)27, (int8_t)6, 
    (int8_t)27, (int8_t)6, (int8_t)27, (int8_t)6, (int8_t)20, 
    (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)8, (int8_t)8, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)8, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)23, 
    (int8_t)6, (int8_t)6, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)6, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)8, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)8, (int8_t)6, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)6, (int8_t)6, (int8_t)8, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)8, (int8_t)8, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)23, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)23, 
    (int8_t)23, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)12, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)20, (int8_t)21, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, (int8_t)8, 
    (int8_t)8, (int8_t)6, (int8_t)8, (int8_t)8, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)25, (int8_t)23, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)19, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)16, (int8_t)16, (int8_t)16, (int8_t)16, (int8_t)0, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)4, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)6, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, 
    (int8_t)1, (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)1, 
    (int8_t)2, (int8_t)1, (int8_t)2, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)0, 
    (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, 
    (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)0, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)0, (int8_t)1, (int8_t)0, 
    (int8_t)1, (int8_t)0, (int8_t)1, (int8_t)0, (int8_t)1, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)0, (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, 
    (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, 
    (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)3, (int8_t)3, (int8_t)3, 
    (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, (int8_t)3, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)3, (int8_t)26, (int8_t)2, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)3, (int8_t)26, 
    (int8_t)26, (int8_t)26, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)2, (int8_t)2, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)0, 
    (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)26, (int8_t)26, (int8_t)26, (int8_t)0, 
    (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, 
    (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)3, (int8_t)26, (int8_t)26, (int8_t)0, 
    (int8_t)12, (int8_t)12, (int8_t)12, (int8_t)12, (int8_t)12, 
    (int8_t)12, (int8_t)12, (int8_t)12, (int8_t)12, (int8_t)12, 
    (int8_t)12, (int8_t)12, (int8_t)16, (int8_t)16, (int8_t)16, 
    (int8_t)16, (int8_t)19, (int8_t)19, (int8_t)19, (int8_t)19, 
    (int8_t)19, (int8_t)19, (int8_t)23, (int8_t)23, (int8_t)28, 
    (int8_t)29, (int8_t)20, (int8_t)28, (int8_t)28, (int8_t)29, 
    (int8_t)20, (int8_t)28, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)13, (int8_t)14, (int8_t)16, (int8_t)16, (int8_t)16, 
    (int8_t)16, (int8_t)16, (int8_t)12, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)28, (int8_t)29, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)22, (int8_t)22, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)24, (int8_t)20, 
    (int8_t)21, (int8_t)0, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)16, (int8_t)16, (int8_t)16, (int8_t)16, 
    (int8_t)16, (int8_t)16, (int8_t)11, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)20, (int8_t)21, (int8_t)2, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)20, (int8_t)21, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, 
    (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, 
    (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, (int8_t)25, 
    (int8_t)25, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)7, (int8_t)7, (int8_t)7, (int8_t)7, 
    (int8_t)6, (int8_t)7, (int8_t)7, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)1, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)1, (int8_t)27, 
    (int8_t)27, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)2, (int8_t)2, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)2, (int8_t)27, (int8_t)1, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)1, (int8_t)27, (int8_t)1, 
    (int8_t)27, (int8_t)1, (int8_t)27, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)27, (int8_t)2, (int8_t)1, 
    (int8_t)1, (int8_t)27, (int8_t)1, (int8_t)2, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)2, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)24, (int8_t)24, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)27, 
    (int8_t)24, (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)24, 
    (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)24, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)24, 
    (int8_t)24, (int8_t)24, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)24, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)20, (int8_t)21, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)11, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)24, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)27, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)12, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)27, (int8_t)4, (int8_t)5, (int8_t)10, (int8_t)20, 
    (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, 
    (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)27, 
    (int8_t)27, (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, 
    (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)19, 
    (int8_t)20, (int8_t)21, (int8_t)21, (int8_t)27, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)6, (int8_t)19, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)27, (int8_t)27, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, 
    (int8_t)26, (int8_t)26, (int8_t)4, (int8_t)4, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)22, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, 
    (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)11, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)0, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)0, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)27, (int8_t)0, (int8_t)27, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)27, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)6, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)24, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)20, (int8_t)21, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, (int8_t)6, 
    (int8_t)6, (int8_t)6, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)23, 
    (int8_t)19, (int8_t)19, (int8_t)22, (int8_t)22, (int8_t)20, 
    (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, 
    (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)20, 
    (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)23, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)22, (int8_t)22, 
    (int8_t)22, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)0, 
    (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)19, 
    (int8_t)20, (int8_t)21, (int8_t)20, (int8_t)21, (int8_t)20, 
    (int8_t)21, (int8_t)23, (int8_t)23, (int8_t)23, (int8_t)24, 
    (int8_t)19, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)0, 
    (int8_t)23, (int8_t)25, (int8_t)23, (int8_t)23, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)0, (int8_t)5, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, 
    (int8_t)0, (int8_t)16, (int8_t)0, (int8_t)23, (int8_t)23, 
    (int8_t)23, (int8_t)25, (int8_t)23, (int8_t)23, (int8_t)23, 
    (int8_t)20, (int8_t)21, (int8_t)23, (int8_t)24, (int8_t)23, 
    (int8_t)19, (int8_t)23, (int8_t)23, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)23, (int8_t)23, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)23, (int8_t)23, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)20, (int8_t)23, (int8_t)21, (int8_t)26, 
    (int8_t)22, (int8_t)26, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)20, (int8_t)24, 
    (int8_t)21, (int8_t)24, (int8_t)0, (int8_t)0, (int8_t)23, 
    (int8_t)20, (int8_t)21, (int8_t)23, (int8_t)22, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)4, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)4, (int8_t)4, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)25, (int8_t)25, (int8_t)24, (int8_t)26, 
    (int8_t)27, (int8_t)25, (int8_t)25, (int8_t)0, (int8_t)27, 
    (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)24, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)16, (int8_t)16, (int8_t)16, (int8_t)27, 
    (int8_t)27, (int8_t)0, (int8_t)0 };

static const int32_t offsetCount = 10234;

static const uint16_t fCharDirIndices[] = {
    (uint16_t)0, (uint16_t)127, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)318, (uint16_t)446, 
    (uint16_t)573, (uint16_t)247, (uint16_t)698, 
    (uint16_t)247, (uint16_t)816, (uint16_t)933, 
    (uint16_t)1046, (uint16_t)1174, (uint16_t)1302, 
    (uint16_t)247, (uint16_t)247, (uint16_t)1429, 
    (uint16_t)1556, (uint16_t)1682, (uint16_t)1793, 
    (uint16_t)1869, (uint16_t)1953, (uint16_t)2031, 
    (uint16_t)2118, (uint16_t)2196, (uint16_t)2274, 
    (uint16_t)2361, (uint16_t)2440, (uint16_t)2544, 
    (uint16_t)2666, (uint16_t)2749, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)2877, 
    (uint16_t)247, (uint16_t)2950, (uint16_t)3078, 
    (uint16_t)3165, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)3232, 
    (uint16_t)3360, (uint16_t)3488, (uint16_t)3616, 
    (uint16_t)3728, (uint16_t)3856, (uint16_t)3876, 
    (uint16_t)3936, (uint16_t)4061, (uint16_t)4189, 
    (uint16_t)4289, (uint16_t)4417, (uint16_t)4523, 
    (uint16_t)4651, (uint16_t)247, (uint16_t)4778, 
    (uint16_t)4896, (uint16_t)4417, (uint16_t)4417, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)5024, 
    (uint16_t)4417, (uint16_t)5054, (uint16_t)5182, 
    (uint16_t)5285, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)5409, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)247, (uint16_t)247, (uint16_t)247, 
    (uint16_t)5508, (uint16_t)5588, (uint16_t)5671, 
    (uint16_t)5671, (uint16_t)5737, (uint16_t)5849, 
    (uint16_t)5973, (uint16_t)6091, (uint16_t)6219, 
    (uint16_t)6321 };

static const int8_t fCharDirValues[] = {
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)8, (int8_t)7, 
    (int8_t)8, (int8_t)9, (int8_t)7, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)7, (int8_t)7, (int8_t)7, 
    (int8_t)8, (int8_t)9, (int8_t)10, (int8_t)10, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)4, (int8_t)6, (int8_t)4, 
    (int8_t)6, (int8_t)3, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)6, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)7, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)6, (int8_t)10, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)4, 
    (int8_t)4, (int8_t)2, (int8_t)2, (int8_t)10, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)2, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)1, (int8_t)17, (int8_t)1, 
    (int8_t)17, (int8_t)17, (int8_t)1, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)6, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)13, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)13, (int8_t)0, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)5, 
    (int8_t)5, (int8_t)5, (int8_t)5, (int8_t)4, (int8_t)5, 
    (int8_t)5, (int8_t)13, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)13, (int8_t)13, (int8_t)17, (int8_t)17, 
    (int8_t)10, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)18, (int8_t)13, 
    (int8_t)17, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)4, 
    (int8_t)4, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)4, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)17, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)17, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)9, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)4, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)18, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, (int8_t)9, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)0, (int8_t)1, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)9, 
    (int8_t)7, (int8_t)11, (int8_t)14, (int8_t)16, (int8_t)12, 
    (int8_t)15, (int8_t)9, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)18, 
    (int8_t)18, (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)4, (int8_t)4, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)4, (int8_t)4, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)4, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)0, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)4, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)4, (int8_t)4, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)2, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)9, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)10, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)1, (int8_t)17, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)4, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)1, 
    (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)0, (int8_t)1, 
    (int8_t)1, (int8_t)0, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, (int8_t)1, 
    (int8_t)1, (int8_t)1, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)0, 
    (int8_t)0, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)17, 
    (int8_t)17, (int8_t)17, (int8_t)17, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)6, (int8_t)10, (int8_t)6, 
    (int8_t)0, (int8_t)10, (int8_t)6, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)4, (int8_t)10, (int8_t)10, 
    (int8_t)4, (int8_t)4, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)10, (int8_t)4, (int8_t)4, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)0, (int8_t)13, (int8_t)0, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, (int8_t)13, 
    (int8_t)0, (int8_t)0, (int8_t)18, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)4, (int8_t)4, (int8_t)4, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)4, 
    (int8_t)6, (int8_t)4, (int8_t)6, (int8_t)3, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, 
    (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)2, (int8_t)6, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)0, (int8_t)0, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)4, (int8_t)4, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)4, (int8_t)4, (int8_t)0, (int8_t)10, 
    (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, (int8_t)0, 
    (int8_t)0, (int8_t)18, (int8_t)18, (int8_t)18, (int8_t)10, 
    (int8_t)10, (int8_t)0, (int8_t)0 };
static const int32_t fCharDirCount = 6449;

static const uint16_t caseIndex[] = {
    (uint16_t)0, (uint16_t)128, (uint16_t)256, (uint16_t)384, 
    (uint16_t)512, (uint16_t)640, (uint16_t)768, 
    (uint16_t)896, (uint16_t)1024, (uint16_t)1152, 
    (uint16_t)1280, (uint16_t)1408, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)1536, 
    (uint16_t)1664, (uint16_t)1792, (uint16_t)1920, 
    (uint16_t)768, (uint16_t)768, (uint16_t)2048, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)768, 
    (uint16_t)768, (uint16_t)768, (uint16_t)2176, 
    (uint16_t)768 };

static const int16_t caseValues[] = {
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)97, 
    (int16_t)98, (int16_t)99, (int16_t)100, (int16_t)101, (int16_t)102, 
    (int16_t)103, (int16_t)104, (int16_t)105, (int16_t)106, (int16_t)107, 
    (int16_t)108, (int16_t)109, (int16_t)110, (int16_t)111, (int16_t)112, 
    (int16_t)113, (int16_t)114, (int16_t)115, (int16_t)116, (int16_t)117, 
    (int16_t)118, (int16_t)119, (int16_t)120, (int16_t)121, (int16_t)122, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)65, (int16_t)66, (int16_t)67, (int16_t)68, 
    (int16_t)69, (int16_t)70, (int16_t)71, (int16_t)72, (int16_t)73, 
    (int16_t)74, (int16_t)75, (int16_t)76, (int16_t)77, (int16_t)78, 
    (int16_t)79, (int16_t)80, (int16_t)81, (int16_t)82, (int16_t)83, 
    (int16_t)84, (int16_t)85, (int16_t)86, (int16_t)87, (int16_t)88, 
    (int16_t)89, (int16_t)90, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)924, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)224, (int16_t)225, (int16_t)226, (int16_t)227, 
    (int16_t)228, (int16_t)229, (int16_t)230, (int16_t)231, (int16_t)232, 
    (int16_t)233, (int16_t)234, (int16_t)235, (int16_t)236, (int16_t)237, 
    (int16_t)238, (int16_t)239, (int16_t)240, (int16_t)241, (int16_t)242, 
    (int16_t)243, (int16_t)244, (int16_t)245, (int16_t)246, (int16_t)-1, 
    (int16_t)248, (int16_t)249, (int16_t)250, (int16_t)251, (int16_t)252, 
    (int16_t)253, (int16_t)254, (int16_t)-1, (int16_t)192, (int16_t)193, 
    (int16_t)194, (int16_t)195, (int16_t)196, (int16_t)197, (int16_t)198, 
    (int16_t)199, (int16_t)200, (int16_t)201, (int16_t)202, (int16_t)203, 
    (int16_t)204, (int16_t)205, (int16_t)206, (int16_t)207, (int16_t)208, 
    (int16_t)209, (int16_t)210, (int16_t)211, (int16_t)212, (int16_t)213, 
    (int16_t)214, (int16_t)-1, (int16_t)216, (int16_t)217, (int16_t)218, 
    (int16_t)219, (int16_t)220, (int16_t)221, (int16_t)222, (int16_t)376, 
    (int16_t)257, (int16_t)256, (int16_t)259, (int16_t)258, (int16_t)261, 
    (int16_t)260, (int16_t)263, (int16_t)262, (int16_t)265, (int16_t)264, 
    (int16_t)267, (int16_t)266, (int16_t)269, (int16_t)268, (int16_t)271, 
    (int16_t)270, (int16_t)273, (int16_t)272, (int16_t)275, (int16_t)274, 
    (int16_t)277, (int16_t)276, (int16_t)279, (int16_t)278, (int16_t)281, 
    (int16_t)280, (int16_t)283, (int16_t)282, (int16_t)285, (int16_t)284, 
    (int16_t)287, (int16_t)286, (int16_t)289, (int16_t)288, (int16_t)291, 
    (int16_t)290, (int16_t)293, (int16_t)292, (int16_t)295, (int16_t)294, 
    (int16_t)297, (int16_t)296, (int16_t)299, (int16_t)298, (int16_t)301, 
    (int16_t)300, (int16_t)303, (int16_t)302, (int16_t)105, (int16_t)73, 
    (int16_t)307, (int16_t)306, (int16_t)309, (int16_t)308, (int16_t)311, 
    (int16_t)310, (int16_t)-1, (int16_t)314, (int16_t)313, (int16_t)316, 
    (int16_t)315, (int16_t)318, (int16_t)317, (int16_t)320, (int16_t)319, 
    (int16_t)322, (int16_t)321, (int16_t)324, (int16_t)323, (int16_t)326, 
    (int16_t)325, (int16_t)328, (int16_t)327, (int16_t)-1, (int16_t)331, 
    (int16_t)330, (int16_t)333, (int16_t)332, (int16_t)335, (int16_t)334, 
    (int16_t)337, (int16_t)336, (int16_t)339, (int16_t)338, (int16_t)341, 
    (int16_t)340, (int16_t)343, (int16_t)342, (int16_t)345, (int16_t)344, 
    (int16_t)347, (int16_t)346, (int16_t)349, (int16_t)348, (int16_t)351, 
    (int16_t)350, (int16_t)353, (int16_t)352, (int16_t)355, (int16_t)354, 
    (int16_t)357, (int16_t)356, (int16_t)359, (int16_t)358, (int16_t)361, 
    (int16_t)360, (int16_t)363, (int16_t)362, (int16_t)365, (int16_t)364, 
    (int16_t)367, (int16_t)366, (int16_t)369, (int16_t)368, (int16_t)371, 
    (int16_t)370, (int16_t)373, (int16_t)372, (int16_t)375, (int16_t)374, 
    (int16_t)255, (int16_t)378, (int16_t)377, (int16_t)380, (int16_t)379, 
    (int16_t)382, (int16_t)381, (int16_t)83, (int16_t)-1, (int16_t)595, 
    (int16_t)387, (int16_t)386, (int16_t)389, (int16_t)388, (int16_t)596, 
    (int16_t)392, (int16_t)391, (int16_t)598, (int16_t)599, (int16_t)396, 
    (int16_t)395, (int16_t)-1, (int16_t)477, (int16_t)601, (int16_t)603, 
    (int16_t)402, (int16_t)401, (int16_t)608, (int16_t)611, (int16_t)502, 
    (int16_t)617, (int16_t)616, (int16_t)409, (int16_t)408, (int16_t)-1, 
    (int16_t)-1, (int16_t)623, (int16_t)626, (int16_t)-1, (int16_t)629, 
    (int16_t)417, (int16_t)416, (int16_t)419, (int16_t)418, (int16_t)421, 
    (int16_t)420, (int16_t)640, (int16_t)424, (int16_t)423, (int16_t)643, 
    (int16_t)-1, (int16_t)-1, (int16_t)429, (int16_t)428, (int16_t)648, 
    (int16_t)432, (int16_t)431, (int16_t)650, (int16_t)651, (int16_t)436, 
    (int16_t)435, (int16_t)438, (int16_t)437, (int16_t)658, (int16_t)441, 
    (int16_t)440, (int16_t)-1, (int16_t)-1, (int16_t)445, (int16_t)444, 
    (int16_t)-1, (int16_t)503, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)454, (int16_t)-1, (int16_t)452, (int16_t)457, 
    (int16_t)-1, (int16_t)455, (int16_t)460, (int16_t)-1, (int16_t)458, 
    (int16_t)462, (int16_t)461, (int16_t)464, (int16_t)463, (int16_t)466, 
    (int16_t)465, (int16_t)468, (int16_t)467, (int16_t)470, (int16_t)469, 
    (int16_t)472, (int16_t)471, (int16_t)474, (int16_t)473, (int16_t)476, 
    (int16_t)475, (int16_t)398, (int16_t)479, (int16_t)478, (int16_t)481, 
    (int16_t)480, (int16_t)483, (int16_t)482, (int16_t)485, (int16_t)484, 
    (int16_t)487, (int16_t)486, (int16_t)489, (int16_t)488, (int16_t)491, 
    (int16_t)490, (int16_t)493, (int16_t)492, (int16_t)495, (int16_t)494, 
    (int16_t)-1, (int16_t)499, (int16_t)-1, (int16_t)497, (int16_t)501, 
    (int16_t)500, (int16_t)405, (int16_t)447, (int16_t)505, (int16_t)504, 
    (int16_t)507, (int16_t)506, (int16_t)509, (int16_t)508, (int16_t)511, 
    (int16_t)510, (int16_t)513, (int16_t)512, (int16_t)515, (int16_t)514, 
    (int16_t)517, (int16_t)516, (int16_t)519, (int16_t)518, (int16_t)521, 
    (int16_t)520, (int16_t)523, (int16_t)522, (int16_t)525, (int16_t)524, 
    (int16_t)527, (int16_t)526, (int16_t)529, (int16_t)528, (int16_t)531, 
    (int16_t)530, (int16_t)533, (int16_t)532, (int16_t)535, (int16_t)534, 
    (int16_t)537, (int16_t)536, (int16_t)539, (int16_t)538, (int16_t)541, 
    (int16_t)540, (int16_t)543, (int16_t)542, (int16_t)-1, (int16_t)-1, 
    (int16_t)547, (int16_t)546, (int16_t)549, (int16_t)548, (int16_t)551, 
    (int16_t)550, (int16_t)553, (int16_t)552, (int16_t)555, (int16_t)554, 
    (int16_t)557, (int16_t)556, (int16_t)559, (int16_t)558, (int16_t)561, 
    (int16_t)560, (int16_t)563, (int16_t)562, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)385, 
    (int16_t)390, (int16_t)-1, (int16_t)393, (int16_t)394, (int16_t)-1, 
    (int16_t)399, (int16_t)-1, (int16_t)400, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)403, (int16_t)-1, (int16_t)-1, 
    (int16_t)404, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)407, (int16_t)406, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)412, (int16_t)-1, (int16_t)-1, 
    (int16_t)413, (int16_t)-1, (int16_t)-1, (int16_t)415, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)422, 
    (int16_t)-1, (int16_t)-1, (int16_t)425, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)430, (int16_t)-1, (int16_t)433, 
    (int16_t)434, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)439, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)940, (int16_t)-1, (int16_t)941, (int16_t)942, 
    (int16_t)943, (int16_t)-1, (int16_t)972, (int16_t)-1, (int16_t)973, 
    (int16_t)974, (int16_t)-1, (int16_t)945, (int16_t)946, (int16_t)947, 
    (int16_t)948, (int16_t)949, (int16_t)950, (int16_t)951, (int16_t)952, 
    (int16_t)953, (int16_t)954, (int16_t)955, (int16_t)956, (int16_t)957, 
    (int16_t)958, (int16_t)959, (int16_t)960, (int16_t)961, (int16_t)-1, 
    (int16_t)963, (int16_t)964, (int16_t)965, (int16_t)966, (int16_t)967, 
    (int16_t)968, (int16_t)969, (int16_t)970, (int16_t)971, (int16_t)902, 
    (int16_t)904, (int16_t)905, (int16_t)906, (int16_t)-1, (int16_t)913, 
    (int16_t)914, (int16_t)915, (int16_t)916, (int16_t)917, (int16_t)918, 
    (int16_t)919, (int16_t)920, (int16_t)921, (int16_t)922, (int16_t)923, 
    (int16_t)924, (int16_t)925, (int16_t)926, (int16_t)927, (int16_t)928, 
    (int16_t)929, (int16_t)931, (int16_t)931, (int16_t)932, (int16_t)933, 
    (int16_t)934, (int16_t)935, (int16_t)936, (int16_t)937, (int16_t)938, 
    (int16_t)939, (int16_t)908, (int16_t)910, (int16_t)911, (int16_t)-1, 
    (int16_t)914, (int16_t)920, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)934, (int16_t)928, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)987, (int16_t)986, (int16_t)989, (int16_t)988, (int16_t)991, 
    (int16_t)990, (int16_t)993, (int16_t)992, (int16_t)995, (int16_t)994, 
    (int16_t)997, (int16_t)996, (int16_t)999, (int16_t)998, (int16_t)1001, 
    (int16_t)1000, (int16_t)1003, (int16_t)1002, (int16_t)1005, (int16_t)1004, 
    (int16_t)1007, (int16_t)1006, (int16_t)922, (int16_t)929, (int16_t)931, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)1104, (int16_t)1105, 
    (int16_t)1106, (int16_t)1107, (int16_t)1108, (int16_t)1109, (int16_t)1110, 
    (int16_t)1111, (int16_t)1112, (int16_t)1113, (int16_t)1114, (int16_t)1115, 
    (int16_t)1116, (int16_t)1117, (int16_t)1118, (int16_t)1119, (int16_t)1072, 
    (int16_t)1073, (int16_t)1074, (int16_t)1075, (int16_t)1076, (int16_t)1077, 
    (int16_t)1078, (int16_t)1079, (int16_t)1080, (int16_t)1081, (int16_t)1082, 
    (int16_t)1083, (int16_t)1084, (int16_t)1085, (int16_t)1086, (int16_t)1087, 
    (int16_t)1088, (int16_t)1089, (int16_t)1090, (int16_t)1091, (int16_t)1092, 
    (int16_t)1093, (int16_t)1094, (int16_t)1095, (int16_t)1096, (int16_t)1097, 
    (int16_t)1098, (int16_t)1099, (int16_t)1100, (int16_t)1101, (int16_t)1102, 
    (int16_t)1103, (int16_t)1040, (int16_t)1041, (int16_t)1042, (int16_t)1043, 
    (int16_t)1044, (int16_t)1045, (int16_t)1046, (int16_t)1047, (int16_t)1048, 
    (int16_t)1049, (int16_t)1050, (int16_t)1051, (int16_t)1052, (int16_t)1053, 
    (int16_t)1054, (int16_t)1055, (int16_t)1056, (int16_t)1057, (int16_t)1058, 
    (int16_t)1059, (int16_t)1060, (int16_t)1061, (int16_t)1062, (int16_t)1063, 
    (int16_t)1064, (int16_t)1065, (int16_t)1066, (int16_t)1067, (int16_t)1068, 
    (int16_t)1069, (int16_t)1070, (int16_t)1071, (int16_t)1024, (int16_t)1025, 
    (int16_t)1026, (int16_t)1027, (int16_t)1028, (int16_t)1029, (int16_t)1030, 
    (int16_t)1031, (int16_t)1032, (int16_t)1033, (int16_t)1034, (int16_t)1035, 
    (int16_t)1036, (int16_t)1037, (int16_t)1038, (int16_t)1039, (int16_t)1121, 
    (int16_t)1120, (int16_t)1123, (int16_t)1122, (int16_t)1125, (int16_t)1124, 
    (int16_t)1127, (int16_t)1126, (int16_t)1129, (int16_t)1128, (int16_t)1131, 
    (int16_t)1130, (int16_t)1133, (int16_t)1132, (int16_t)1135, (int16_t)1134, 
    (int16_t)1137, (int16_t)1136, (int16_t)1139, (int16_t)1138, (int16_t)1141, 
    (int16_t)1140, (int16_t)1143, (int16_t)1142, (int16_t)1145, (int16_t)1144, 
    (int16_t)1147, (int16_t)1146, (int16_t)1149, (int16_t)1148, (int16_t)1151, 
    (int16_t)1150, (int16_t)1153, (int16_t)1152, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)1165, (int16_t)1164, 
    (int16_t)1167, (int16_t)1166, (int16_t)1169, (int16_t)1168, (int16_t)1171, 
    (int16_t)1170, (int16_t)1173, (int16_t)1172, (int16_t)1175, (int16_t)1174, 
    (int16_t)1177, (int16_t)1176, (int16_t)1179, (int16_t)1178, (int16_t)1181, 
    (int16_t)1180, (int16_t)1183, (int16_t)1182, (int16_t)1185, (int16_t)1184, 
    (int16_t)1187, (int16_t)1186, (int16_t)1189, (int16_t)1188, (int16_t)1191, 
    (int16_t)1190, (int16_t)1193, (int16_t)1192, (int16_t)1195, (int16_t)1194, 
    (int16_t)1197, (int16_t)1196, (int16_t)1199, (int16_t)1198, (int16_t)1201, 
    (int16_t)1200, (int16_t)1203, (int16_t)1202, (int16_t)1205, (int16_t)1204, 
    (int16_t)1207, (int16_t)1206, (int16_t)1209, (int16_t)1208, (int16_t)1211, 
    (int16_t)1210, (int16_t)1213, (int16_t)1212, (int16_t)1215, (int16_t)1214, 
    (int16_t)-1, (int16_t)1218, (int16_t)1217, (int16_t)1220, (int16_t)1219, 
    (int16_t)-1, (int16_t)-1, (int16_t)1224, (int16_t)1223, (int16_t)-1, 
    (int16_t)-1, (int16_t)1228, (int16_t)1227, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)1233, (int16_t)1232, (int16_t)1235, (int16_t)1234, 
    (int16_t)1237, (int16_t)1236, (int16_t)1239, (int16_t)1238, (int16_t)1241, 
    (int16_t)1240, (int16_t)1243, (int16_t)1242, (int16_t)1245, (int16_t)1244, 
    (int16_t)1247, (int16_t)1246, (int16_t)1249, (int16_t)1248, (int16_t)1251, 
    (int16_t)1250, (int16_t)1253, (int16_t)1252, (int16_t)1255, (int16_t)1254, 
    (int16_t)1257, (int16_t)1256, (int16_t)1259, (int16_t)1258, (int16_t)1261, 
    (int16_t)1260, (int16_t)1263, (int16_t)1262, (int16_t)1265, (int16_t)1264, 
    (int16_t)1267, (int16_t)1266, (int16_t)1269, (int16_t)1268, (int16_t)-1, 
    (int16_t)-1, (int16_t)1273, (int16_t)1272, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)1377, (int16_t)1378, 
    (int16_t)1379, (int16_t)1380, (int16_t)1381, (int16_t)1382, (int16_t)1383, 
    (int16_t)1384, (int16_t)1385, (int16_t)1386, (int16_t)1387, (int16_t)1388, 
    (int16_t)1389, (int16_t)1390, (int16_t)1391, (int16_t)1392, (int16_t)1393, 
    (int16_t)1394, (int16_t)1395, (int16_t)1396, (int16_t)1397, (int16_t)1398, 
    (int16_t)1399, (int16_t)1400, (int16_t)1401, (int16_t)1402, (int16_t)1403, 
    (int16_t)1404, (int16_t)1405, (int16_t)1406, (int16_t)1407, (int16_t)1408, 
    (int16_t)1409, (int16_t)1410, (int16_t)1411, (int16_t)1412, (int16_t)1413, 
    (int16_t)1414, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)1329, (int16_t)1330, (int16_t)1331, (int16_t)1332, 
    (int16_t)1333, (int16_t)1334, (int16_t)1335, (int16_t)1336, (int16_t)1337, 
    (int16_t)1338, (int16_t)1339, (int16_t)1340, (int16_t)1341, (int16_t)1342, 
    (int16_t)1343, (int16_t)1344, (int16_t)1345, (int16_t)1346, (int16_t)1347, 
    (int16_t)1348, (int16_t)1349, (int16_t)1350, (int16_t)1351, (int16_t)1352, 
    (int16_t)1353, (int16_t)1354, (int16_t)1355, (int16_t)1356, (int16_t)1357, 
    (int16_t)1358, (int16_t)1359, (int16_t)1360, (int16_t)1361, (int16_t)1362, 
    (int16_t)1363, (int16_t)1364, (int16_t)1365, (int16_t)1366, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)7681, (int16_t)7680, (int16_t)7683, (int16_t)7682, (int16_t)7685, 
    (int16_t)7684, (int16_t)7687, (int16_t)7686, (int16_t)7689, (int16_t)7688, 
    (int16_t)7691, (int16_t)7690, (int16_t)7693, (int16_t)7692, (int16_t)7695, 
    (int16_t)7694, (int16_t)7697, (int16_t)7696, (int16_t)7699, (int16_t)7698, 
    (int16_t)7701, (int16_t)7700, (int16_t)7703, (int16_t)7702, (int16_t)7705, 
    (int16_t)7704, (int16_t)7707, (int16_t)7706, (int16_t)7709, (int16_t)7708, 
    (int16_t)7711, (int16_t)7710, (int16_t)7713, (int16_t)7712, (int16_t)7715, 
    (int16_t)7714, (int16_t)7717, (int16_t)7716, (int16_t)7719, (int16_t)7718, 
    (int16_t)7721, (int16_t)7720, (int16_t)7723, (int16_t)7722, (int16_t)7725, 
    (int16_t)7724, (int16_t)7727, (int16_t)7726, (int16_t)7729, (int16_t)7728, 
    (int16_t)7731, (int16_t)7730, (int16_t)7733, (int16_t)7732, (int16_t)7735, 
    (int16_t)7734, (int16_t)7737, (int16_t)7736, (int16_t)7739, (int16_t)7738, 
    (int16_t)7741, (int16_t)7740, (int16_t)7743, (int16_t)7742, (int16_t)7745, 
    (int16_t)7744, (int16_t)7747, (int16_t)7746, (int16_t)7749, (int16_t)7748, 
    (int16_t)7751, (int16_t)7750, (int16_t)7753, (int16_t)7752, (int16_t)7755, 
    (int16_t)7754, (int16_t)7757, (int16_t)7756, (int16_t)7759, (int16_t)7758, 
    (int16_t)7761, (int16_t)7760, (int16_t)7763, (int16_t)7762, (int16_t)7765, 
    (int16_t)7764, (int16_t)7767, (int16_t)7766, (int16_t)7769, (int16_t)7768, 
    (int16_t)7771, (int16_t)7770, (int16_t)7773, (int16_t)7772, (int16_t)7775, 
    (int16_t)7774, (int16_t)7777, (int16_t)7776, (int16_t)7779, (int16_t)7778, 
    (int16_t)7781, (int16_t)7780, (int16_t)7783, (int16_t)7782, (int16_t)7785, 
    (int16_t)7784, (int16_t)7787, (int16_t)7786, (int16_t)7789, (int16_t)7788, 
    (int16_t)7791, (int16_t)7790, (int16_t)7793, (int16_t)7792, (int16_t)7795, 
    (int16_t)7794, (int16_t)7797, (int16_t)7796, (int16_t)7799, (int16_t)7798, 
    (int16_t)7801, (int16_t)7800, (int16_t)7803, (int16_t)7802, (int16_t)7805, 
    (int16_t)7804, (int16_t)7807, (int16_t)7806, (int16_t)7809, (int16_t)7808, 
    (int16_t)7811, (int16_t)7810, (int16_t)7813, (int16_t)7812, (int16_t)7815, 
    (int16_t)7814, (int16_t)7817, (int16_t)7816, (int16_t)7819, (int16_t)7818, 
    (int16_t)7821, (int16_t)7820, (int16_t)7823, (int16_t)7822, (int16_t)7825, 
    (int16_t)7824, (int16_t)7827, (int16_t)7826, (int16_t)7829, (int16_t)7828, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)7776, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)7841, (int16_t)7840, (int16_t)7843, (int16_t)7842, (int16_t)7845, 
    (int16_t)7844, (int16_t)7847, (int16_t)7846, (int16_t)7849, (int16_t)7848, 
    (int16_t)7851, (int16_t)7850, (int16_t)7853, (int16_t)7852, (int16_t)7855, 
    (int16_t)7854, (int16_t)7857, (int16_t)7856, (int16_t)7859, (int16_t)7858, 
    (int16_t)7861, (int16_t)7860, (int16_t)7863, (int16_t)7862, (int16_t)7865, 
    (int16_t)7864, (int16_t)7867, (int16_t)7866, (int16_t)7869, (int16_t)7868, 
    (int16_t)7871, (int16_t)7870, (int16_t)7873, (int16_t)7872, (int16_t)7875, 
    (int16_t)7874, (int16_t)7877, (int16_t)7876, (int16_t)7879, (int16_t)7878, 
    (int16_t)7881, (int16_t)7880, (int16_t)7883, (int16_t)7882, (int16_t)7885, 
    (int16_t)7884, (int16_t)7887, (int16_t)7886, (int16_t)7889, (int16_t)7888, 
    (int16_t)7891, (int16_t)7890, (int16_t)7893, (int16_t)7892, (int16_t)7895, 
    (int16_t)7894, (int16_t)7897, (int16_t)7896, (int16_t)7899, (int16_t)7898, 
    (int16_t)7901, (int16_t)7900, (int16_t)7903, (int16_t)7902, (int16_t)7905, 
    (int16_t)7904, (int16_t)7907, (int16_t)7906, (int16_t)7909, (int16_t)7908, 
    (int16_t)7911, (int16_t)7910, (int16_t)7913, (int16_t)7912, (int16_t)7915, 
    (int16_t)7914, (int16_t)7917, (int16_t)7916, (int16_t)7919, (int16_t)7918, 
    (int16_t)7921, (int16_t)7920, (int16_t)7923, (int16_t)7922, (int16_t)7925, 
    (int16_t)7924, (int16_t)7927, (int16_t)7926, (int16_t)7929, (int16_t)7928, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)7944, (int16_t)7945, (int16_t)7946, (int16_t)7947, 
    (int16_t)7948, (int16_t)7949, (int16_t)7950, (int16_t)7951, (int16_t)7936, 
    (int16_t)7937, (int16_t)7938, (int16_t)7939, (int16_t)7940, (int16_t)7941, 
    (int16_t)7942, (int16_t)7943, (int16_t)7960, (int16_t)7961, (int16_t)7962, 
    (int16_t)7963, (int16_t)7964, (int16_t)7965, (int16_t)-1, (int16_t)-1, 
    (int16_t)7952, (int16_t)7953, (int16_t)7954, (int16_t)7955, (int16_t)7956, 
    (int16_t)7957, (int16_t)-1, (int16_t)-1, (int16_t)7976, (int16_t)7977, 
    (int16_t)7978, (int16_t)7979, (int16_t)7980, (int16_t)7981, (int16_t)7982, 
    (int16_t)7983, (int16_t)7968, (int16_t)7969, (int16_t)7970, (int16_t)7971, 
    (int16_t)7972, (int16_t)7973, (int16_t)7974, (int16_t)7975, (int16_t)7992, 
    (int16_t)7993, (int16_t)7994, (int16_t)7995, (int16_t)7996, (int16_t)7997, 
    (int16_t)7998, (int16_t)7999, (int16_t)7984, (int16_t)7985, (int16_t)7986, 
    (int16_t)7987, (int16_t)7988, (int16_t)7989, (int16_t)7990, (int16_t)7991, 
    (int16_t)8008, (int16_t)8009, (int16_t)8010, (int16_t)8011, (int16_t)8012, 
    (int16_t)8013, (int16_t)-1, (int16_t)-1, (int16_t)8000, (int16_t)8001, 
    (int16_t)8002, (int16_t)8003, (int16_t)8004, (int16_t)8005, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)8025, (int16_t)-1, (int16_t)8027, 
    (int16_t)-1, (int16_t)8029, (int16_t)-1, (int16_t)8031, (int16_t)-1, 
    (int16_t)8017, (int16_t)-1, (int16_t)8019, (int16_t)-1, (int16_t)8021, 
    (int16_t)-1, (int16_t)8023, (int16_t)8040, (int16_t)8041, (int16_t)8042, 
    (int16_t)8043, (int16_t)8044, (int16_t)8045, (int16_t)8046, (int16_t)8047, 
    (int16_t)8032, (int16_t)8033, (int16_t)8034, (int16_t)8035, (int16_t)8036, 
    (int16_t)8037, (int16_t)8038, (int16_t)8039, (int16_t)8122, (int16_t)8123, 
    (int16_t)8136, (int16_t)8137, (int16_t)8138, (int16_t)8139, (int16_t)8154, 
    (int16_t)8155, (int16_t)8184, (int16_t)8185, (int16_t)8170, (int16_t)8171, 
    (int16_t)8186, (int16_t)8187, (int16_t)-1, (int16_t)-1, (int16_t)8072, 
    (int16_t)8073, (int16_t)8074, (int16_t)8075, (int16_t)8076, (int16_t)8077, 
    (int16_t)8078, (int16_t)8079, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)8088, (int16_t)8089, (int16_t)8090, (int16_t)8091, (int16_t)8092, 
    (int16_t)8093, (int16_t)8094, (int16_t)8095, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)8104, (int16_t)8105, (int16_t)8106, (int16_t)8107, 
    (int16_t)8108, (int16_t)8109, (int16_t)8110, (int16_t)8111, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)8120, (int16_t)8121, (int16_t)-1, 
    (int16_t)8124, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)8112, (int16_t)8113, (int16_t)8048, (int16_t)8049, (int16_t)-1, 
    (int16_t)-1, (int16_t)921, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)8140, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)8050, (int16_t)8051, (int16_t)8052, (int16_t)8053, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)8152, 
    (int16_t)8153, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)8144, (int16_t)8145, (int16_t)8054, 
    (int16_t)8055, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)8168, (int16_t)8169, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)8172, (int16_t)-1, (int16_t)-1, (int16_t)8160, (int16_t)8161, 
    (int16_t)8058, (int16_t)8059, (int16_t)8165, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)8188, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)8056, 
    (int16_t)8057, (int16_t)8060, (int16_t)8061, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)969, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)107, 
    (int16_t)229, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-191, (int16_t)-190, 
    (int16_t)-189, (int16_t)-188, (int16_t)-187, (int16_t)-186, (int16_t)-185, 
    (int16_t)-184, (int16_t)-183, (int16_t)-182, (int16_t)-181, (int16_t)-180, 
    (int16_t)-179, (int16_t)-178, (int16_t)-177, (int16_t)-176, (int16_t)-175, 
    (int16_t)-174, (int16_t)-173, (int16_t)-172, (int16_t)-171, (int16_t)-170, 
    (int16_t)-169, (int16_t)-168, (int16_t)-167, (int16_t)-166, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-223, (int16_t)-222, (int16_t)-221, (int16_t)-220, (int16_t)-219, 
    (int16_t)-218, (int16_t)-217, (int16_t)-216, (int16_t)-215, (int16_t)-214, 
    (int16_t)-213, (int16_t)-212, (int16_t)-211, (int16_t)-210, (int16_t)-209, 
    (int16_t)-208, (int16_t)-207, (int16_t)-206, (int16_t)-205, (int16_t)-204, 
    (int16_t)-203, (int16_t)-202, (int16_t)-201, (int16_t)-200, (int16_t)-199, 
    (int16_t)-198, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, (int16_t)-1, 
    (int16_t)-1, (int16_t)-1, (int16_t)-1 };

static const int32_t caseCount = 2304;

static const UCharDigitPair fCodeDigitTable[] = {
    { 0x0030, 0}, 
    { 0x0031, 1}, 
    { 0x0032, 2}, 
    { 0x0033, 3}, 
    { 0x0034, 4}, 
    { 0x0035, 5}, 
    { 0x0036, 6}, 
    { 0x0037, 7}, 
    { 0x0038, 8}, 
    { 0x0039, 9}, 
    { 0x00B2, 2}, 
    { 0x00B3, 3}, 
    { 0x00B9, 1}, 
    { 0x0660, 0}, 
    { 0x0661, 1}, 
    { 0x0662, 2}, 
    { 0x0663, 3}, 
    { 0x0664, 4}, 
    { 0x0665, 5}, 
    { 0x0666, 6}, 
    { 0x0667, 7}, 
    { 0x0668, 8}, 
    { 0x0669, 9}, 
    { 0x06F0, 0}, 
    { 0x06F1, 1}, 
    { 0x06F2, 2}, 
    { 0x06F3, 3}, 
    { 0x06F4, 4}, 
    { 0x06F5, 5}, 
    { 0x06F6, 6}, 
    { 0x06F7, 7}, 
    { 0x06F8, 8}, 
    { 0x06F9, 9}, 
    { 0x0966, 0}, 
    { 0x0967, 1}, 
    { 0x0968, 2}, 
    { 0x0969, 3}, 
    { 0x096A, 4}, 
    { 0x096B, 5}, 
    { 0x096C, 6}, 
    { 0x096D, 7}, 
    { 0x096E, 8}, 
    { 0x096F, 9}, 
    { 0x09E6, 0}, 
    { 0x09E7, 1}, 
    { 0x09E8, 2}, 
    { 0x09E9, 3}, 
    { 0x09EA, 4}, 
    { 0x09EB, 5}, 
    { 0x09EC, 6}, 
    { 0x09ED, 7}, 
    { 0x09EE, 8}, 
    { 0x09EF, 9}, 
    { 0x0A66, 0}, 
    { 0x0A67, 1}, 
    { 0x0A68, 2}, 
    { 0x0A69, 3}, 
    { 0x0A6A, 4}, 
    { 0x0A6B, 5}, 
    { 0x0A6C, 6}, 
    { 0x0A6D, 7}, 
    { 0x0A6E, 8}, 
    { 0x0A6F, 9}, 
    { 0x0AE6, 0}, 
    { 0x0AE7, 1}, 
    { 0x0AE8, 2}, 
    { 0x0AE9, 3}, 
    { 0x0AEA, 4}, 
    { 0x0AEB, 5}, 
    { 0x0AEC, 6}, 
    { 0x0AED, 7}, 
    { 0x0AEE, 8}, 
    { 0x0AEF, 9}, 
    { 0x0B66, 0}, 
    { 0x0B67, 1}, 
    { 0x0B68, 2}, 
    { 0x0B69, 3}, 
    { 0x0B6A, 4}, 
    { 0x0B6B, 5}, 
    { 0x0B6C, 6}, 
    { 0x0B6D, 7}, 
    { 0x0B6E, 8}, 
    { 0x0B6F, 9}, 
    { 0x0BE7, 1}, 
    { 0x0BE8, 2}, 
    { 0x0BE9, 3}, 
    { 0x0BEA, 4}, 
    { 0x0BEB, 5}, 
    { 0x0BEC, 6}, 
    { 0x0BED, 7}, 
    { 0x0BEE, 8}, 
    { 0x0BEF, 9}, 
    { 0x0C66, 0}, 
    { 0x0C67, 1}, 
    { 0x0C68, 2}, 
    { 0x0C69, 3}, 
    { 0x0C6A, 4}, 
    { 0x0C6B, 5}, 
    { 0x0C6C, 6}, 
    { 0x0C6D, 7}, 
    { 0x0C6E, 8}, 
    { 0x0C6F, 9}, 
    { 0x0CE6, 0}, 
    { 0x0CE7, 1}, 
    { 0x0CE8, 2}, 
    { 0x0CE9, 3}, 
    { 0x0CEA, 4}, 
    { 0x0CEB, 5}, 
    { 0x0CEC, 6}, 
    { 0x0CED, 7}, 
    { 0x0CEE, 8}, 
    { 0x0CEF, 9}, 
    { 0x0D66, 0}, 
    { 0x0D67, 1}, 
    { 0x0D68, 2}, 
    { 0x0D69, 3}, 
    { 0x0D6A, 4}, 
    { 0x0D6B, 5}, 
    { 0x0D6C, 6}, 
    { 0x0D6D, 7}, 
    { 0x0D6E, 8}, 
    { 0x0D6F, 9}, 
    { 0x0E50, 0}, 
    { 0x0E51, 1}, 
    { 0x0E52, 2}, 
    { 0x0E53, 3}, 
    { 0x0E54, 4}, 
    { 0x0E55, 5}, 
    { 0x0E56, 6}, 
    { 0x0E57, 7}, 
    { 0x0E58, 8}, 
    { 0x0E59, 9}, 
    { 0x0ED0, 0}, 
    { 0x0ED1, 1}, 
    { 0x0ED2, 2}, 
    { 0x0ED3, 3}, 
    { 0x0ED4, 4}, 
    { 0x0ED5, 5}, 
    { 0x0ED6, 6}, 
    { 0x0ED7, 7}, 
    { 0x0ED8, 8}, 
    { 0x0ED9, 9}, 
    { 0x0F20, 0}, 
    { 0x0F21, 1}, 
    { 0x0F22, 2}, 
    { 0x0F23, 3}, 
    { 0x0F24, 4}, 
    { 0x0F25, 5}, 
    { 0x0F26, 6}, 
    { 0x0F27, 7}, 
    { 0x0F28, 8}, 
    { 0x0F29, 9}, 
    { 0x1040, 0}, 
    { 0x1041, 1}, 
    { 0x1042, 2}, 
    { 0x1043, 3}, 
    { 0x1044, 4}, 
    { 0x1045, 5}, 
    { 0x1046, 6}, 
    { 0x1047, 7}, 
    { 0x1048, 8}, 
    { 0x1049, 9}, 
    { 0x1369, 1}, 
    { 0x136A, 2}, 
    { 0x136B, 3}, 
    { 0x136C, 4}, 
    { 0x136D, 5}, 
    { 0x136E, 6}, 
    { 0x136F, 7}, 
    { 0x1370, 8}, 
    { 0x1371, 9}, 
    { 0x17E0, 0}, 
    { 0x17E1, 1}, 
    { 0x17E2, 2}, 
    { 0x17E3, 3}, 
    { 0x17E4, 4}, 
    { 0x17E5, 5}, 
    { 0x17E6, 6}, 
    { 0x17E7, 7}, 
    { 0x17E8, 8}, 
    { 0x17E9, 9}, 
    { 0x1810, 0}, 
    { 0x1811, 1}, 
    { 0x1812, 2}, 
    { 0x1813, 3}, 
    { 0x1814, 4}, 
    { 0x1815, 5}, 
    { 0x1816, 6}, 
    { 0x1817, 7}, 
    { 0x1818, 8}, 
    { 0x1819, 9}, 
    { 0x2070, 0}, 
    { 0x2074, 4}, 
    { 0x2075, 5}, 
    { 0x2076, 6}, 
    { 0x2077, 7}, 
    { 0x2078, 8}, 
    { 0x2079, 9}, 
    { 0x2080, 0}, 
    { 0x2081, 1}, 
    { 0x2082, 2}, 
    { 0x2083, 3}, 
    { 0x2084, 4}, 
    { 0x2085, 5}, 
    { 0x2086, 6}, 
    { 0x2087, 7}, 
    { 0x2088, 8}, 
    { 0x2089, 9}, 
    { 0x2460, 1}, 
    { 0x2461, 2}, 
    { 0x2462, 3}, 
    { 0x2463, 4}, 
    { 0x2464, 5}, 
    { 0x2465, 6}, 
    { 0x2466, 7}, 
    { 0x2467, 8}, 
    { 0x2468, 9}, 
    { 0x2474, 1}, 
    { 0x2475, 2}, 
    { 0x2476, 3}, 
    { 0x2477, 4}, 
    { 0x2478, 5}, 
    { 0x2479, 6}, 
    { 0x247A, 7}, 
    { 0x247B, 8}, 
    { 0x247C, 9}, 
    { 0x2488, 1}, 
    { 0x2489, 2}, 
    { 0x248A, 3}, 
    { 0x248B, 4}, 
    { 0x248C, 5}, 
    { 0x248D, 6}, 
    { 0x248E, 7}, 
    { 0x248F, 8}, 
    { 0x2490, 9}, 
    { 0x24EA, 0}, 
    { 0x2776, 1}, 
    { 0x2777, 2}, 
    { 0x2778, 3}, 
    { 0x2779, 4}, 
    { 0x277A, 5}, 
    { 0x277B, 6}, 
    { 0x277C, 7}, 
    { 0x277D, 8}, 
    { 0x277E, 9}, 
    { 0x2780, 1}, 
    { 0x2781, 2}, 
    { 0x2782, 3}, 
    { 0x2783, 4}, 
    { 0x2784, 5}, 
    { 0x2785, 6}, 
    { 0x2786, 7}, 
    { 0x2787, 8}, 
    { 0x2788, 9}, 
    { 0x278A, 1}, 
    { 0x278B, 2}, 
    { 0x278C, 3}, 
    { 0x278D, 4}, 
    { 0x278E, 5}, 
    { 0x278F, 6}, 
    { 0x2790, 7}, 
    { 0x2791, 8}, 
    { 0x2792, 9}, 
    { 0xFF10, 0}, 
    { 0xFF11, 1}, 
    { 0xFF12, 2}, 
    { 0xFF13, 3}, 
    { 0xFF14, 4}, 
    { 0xFF15, 5}, 
    { 0xFF16, 6}, 
    { 0xFF17, 7}, 
    { 0xFF18, 8}, 
    { 0xFF19, 9},
	{ 0x3007, 0}, /* Han Zero*/
    { 0x4E00, 1}, /* Han One*/
    { 0x4E8C, 2}, /* Han Two*/
    { 0x4E09, 3}, /* Han Three*/
    { 0x56D8, 4}, /* Han Four*/
    { 0x4E94, 5}, /* Han Five*/
    { 0x516D, 6}, /* Han Six*/
    { 0x4E03, 7}, /* Han Seven*/
    { 0x516B, 8}, /* Han Eight*/
    { 0x4E5D, 9}, /* Han Nine*/
    { 0xFFFF, 0xF}
};
static const int8_t  isLetterMask = (1 << U_UPPERCASE_LETTER) | (1 << U_LOWERCASE_LETTER) 
                                    | (1 << U_TITLECASE_LETTER) | (1 << U_MODIFIER_LETTER)
                                    | (1 << U_OTHER_LETTER);


static const BlockScriptMap fScriptIndex[] = {
/* Generated from the Unicode-3.0-beta blocks.txt file */
  { 0x0000, 0x007F }, /*BASIC_LATIN */
  { 0x0080, 0x00FF }, /*LATIN_1_SUPPLEMENT */
  { 0x0100, 0x017F }, /*LATIN_EXTENDED_A */
  { 0x0180, 0x024F }, /*LATIN_EXTENDED_B */
  { 0x0250, 0x02AF }, /*IPA_EXTENSIONS */
  { 0x02B0, 0x02FF }, /*SPACING_MODIFIER_LETTERS */
  { 0x0300, 0x036F }, /*COMBINING_DIACRITICAL_MARKS */
  { 0x0370, 0x03FF }, /*GREEK */
  { 0x0400, 0x04FF }, /*CYRILLIC */
  { 0x0530, 0x058F }, /*ARMENIAN */
  { 0x0590, 0x05FF }, /*HEBREW */
  { 0x0600, 0x06FF }, /*ARABIC */
  { 0x0700, 0x074F }, /*SYRIAC */
  { 0x0780, 0x07BF }, /*THAANA */
  { 0x0900, 0x097F }, /*DEVANAGARI */
  { 0x0980, 0x09FF }, /*BENGALI */
  { 0x0A00, 0x0A7F }, /*GURMUKHI */
  { 0x0A80, 0x0AFF }, /*GUJARATI */
  { 0x0B00, 0x0B7F }, /*ORIYA */
  { 0x0B80, 0x0BFF }, /*TAMIL */
  { 0x0C00, 0x0C7F }, /*TELUGU */
  { 0x0C80, 0x0CFF }, /*KANNADA */
  { 0x0D00, 0x0D7F }, /*MALAYALAM */
  { 0x0D80, 0x0DFF }, /*SINHALA */
  { 0x0E00, 0x0E7F }, /*THAI */
  { 0x0E80, 0x0EFF }, /*LAO */
  { 0x0F00, 0x0FFF }, /*TIBETAN */
  { 0x1000, 0x109F }, /*MYANMAR */
  { 0x10A0, 0x10FF }, /*GEORGIAN */
  { 0x1100, 0x11FF }, /*HANGUL_JAMO */
  { 0x1200, 0x137F }, /*ETHIOPIC */
  { 0x13A0, 0x13FF }, /*CHEROKEE */
  { 0x1400, 0x167F }, /*UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS */
  { 0x1680, 0x169F }, /*OGHAM */
  { 0x16A0, 0x16FF }, /*RUNIC */
  { 0x1780, 0x17FF }, /*KHMER */
  { 0x1800, 0x18AF }, /*MONGOLIAN */
  { 0x1E00, 0x1EFF }, /*LATIN_EXTENDED_ADDITIONAL */
  { 0x1F00, 0x1FFF }, /*GREEK_EXTENDED */
  { 0x2000, 0x206F }, /*GENERAL_PUNCTUATION */
  { 0x2070, 0x209F }, /*SUPERSCRIPTS_AND_SUBSCRIPTS */
  { 0x20A0, 0x20CF }, /*CURRENCY_SYMBOLS */
  { 0x20D0, 0x20FF }, /*COMBINING_MARKS_FOR_SYMBOLS */
  { 0x2100, 0x214F }, /*LETTERLIKE_SYMBOLS */
  { 0x2150, 0x218F }, /*NUMBER_FORMS */
  { 0x2190, 0x21FF }, /*ARROWS */
  { 0x2200, 0x22FF }, /*MATHEMATICAL_OPERATORS */
  { 0x2300, 0x23FF }, /*MISCELLANEOUS_TECHNICAL */
  { 0x2400, 0x243F }, /*CONTROL_PICTURES */
  { 0x2440, 0x245F }, /*OPTICAL_CHARACTER_RECOGNITION */
  { 0x2460, 0x24FF }, /*ENCLOSED_ALPHANUMERICS */
  { 0x2500, 0x257F }, /*BOX_DRAWING */
  { 0x2580, 0x259F }, /*BLOCK_ELEMENTS */
  { 0x25A0, 0x25FF }, /*GEOMETRIC_SHAPES */
  { 0x2600, 0x26FF }, /*MISCELLANEOUS_SYMBOLS */
  { 0x2700, 0x27BF }, /*DINGBATS */
  { 0x2800, 0x28FF }, /*BRAILLE_PATTERNS */
  { 0x2E80, 0x2EFF }, /*CJK_RADICALS_SUPPLEMENT */
  { 0x2F00, 0x2FDF }, /*KANGXI_RADICALS */
  { 0x2FF0, 0x2FFF }, /*IDEOGRAPHIC_DESCRIPTION_CHARACTERS */
  { 0x3000, 0x303F }, /*CJK_SYMBOLS_AND_PUNCTUATION */
  { 0x3040, 0x309F }, /*HIRAGANA */
  { 0x30A0, 0x30FF }, /*KATAKANA */
  { 0x3100, 0x312F }, /*BOPOMOFO */
  { 0x3130, 0x318F }, /*HANGUL_COMPATIBILITY_JAMO */
  { 0x3190, 0x319F }, /*KANBUN */
  { 0x31A0, 0x31BF }, /*BOPOMOFO_EXTENDED */
  { 0x3200, 0x32FF }, /*ENCLOSED_CJK_LETTERS_AND_MONTHS */
  { 0x3300, 0x33FF }, /*CJK_COMPATIBILITY */
  { 0x3400, 0x4DB5 }, /*CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A */
  { 0x4E00, 0x9FFF }, /*CJK_UNIFIED_IDEOGRAPHS */
  { 0xA000, 0xA48F }, /*YI_SYLLABLES */
  { 0xA490, 0xA4CF }, /*YI_RADICALS */
  { 0xAC00, 0xD7A3 }, /*HANGUL_SYLLABLES */
  { 0xD800, 0xDB7F }, /*HIGH_SURROGATES */
  { 0xDB80, 0xDBFF }, /*HIGH_PRIVATE_USE_SURROGATES */
  { 0xDC00, 0xDFFF }, /*LOW_SURROGATES */
  { 0xE000, 0xF8FF }, /*PRIVATE_USE */
  { 0xF900, 0xFAFF }, /*CJK_COMPATIBILITY_IDEOGRAPHS */
  { 0xFB00, 0xFB4F }, /*ALPHABETIC_PRESENTATION_FORMS */
  { 0xFB50, 0xFDFF }, /*ARABIC_PRESENTATION_FORMS_A */
  { 0xFE20, 0xFE2F }, /*COMBINING_HALF_MARKS */
  { 0xFE30, 0xFE4F }, /*CJK_COMPATIBILITY_FORMS */
  { 0xFE50, 0xFE6F }, /*SMALL_FORM_VARIANTS */
  { 0xFE70, 0xFEFE }, /*ARABIC_PRESENTATION_FORMS_B */
  { 0xFEFF, 0xFEFF }, /*U_SPECIALS */
  { 0xFF00, 0xFFEF }, /*HALFWIDTH_AND_FULLWIDTH_FORMS */
  { 0xFFF0, 0xFFFD }, /*SPECIALS_2 = "U_CHAR_SCRIPT_COUNT" (really specials) */
  { 0xFFFF, 0xFFFF } /* END */
};
const UChar cellWidthRanges[] =
    {
        0x0000, /* general scripts area*/
        0x1100, /* combining Hangul choseong*/
        0x1160, /* combining Hangul jungseong and jongseong*/
        0x1e00, /* Latin Extended Additional, Greek Extended*/
        0x2000, /* symbols and punctuation*/
        0x3000, /* CJK phonetics & symbols, CJK ideographs, Hangul syllables*/
        0xd800, /* surrogates, private use*/
        0xf900, /* CJK compatibility ideographs*/
        0xfb00, /* alphabetic presentation forms, Arabic presentations forms A, combining half marks*/
        0xfe30, /* CJK compatibility forms, small form variants*/
        0xfe70, /* Arabic presentation forms B*/
        0xff00, /* fullwidth ASCII*/
        0xff60, /* halfwidth, CJK punctuation, Katakana, Hangul Jamo*/
        0xffe0, /* fullwidth punctuation and currency signs*/
        0xffe8, /* halfwidth forms, arrows, and shapes*/
        0xfff0  /* specials*/
    };

const UChar cellWidthValues[] =
    {
        U_HALF_WIDTH,    /* general scripts area*/
        U_FULL_WIDTH,    /* combining Hangul choseong*/
        U_ZERO_WIDTH,    /* combining Hangul jungseong and jongseong*/
        U_HALF_WIDTH,    /* Latin extended aAdditional, Greek extended*/
        U_NEUTRAL_WIDTH, /* symbols and punctuation*/
        U_FULL_WIDTH,    /* CJK phonetics & symbols, CJK ideographs, Hangul syllables*/
        U_NEUTRAL_WIDTH, /* surrogates, private use*/
        U_FULL_WIDTH,    /* CJK compatibility ideographs*/
        U_HALF_WIDTH,    /* alphabetic presentation forms, Arabic presentations forms A, combining half marks*/
        U_FULL_WIDTH,    /* CJK compatibility forms, small form variants*/
        U_HALF_WIDTH,    /* Arabic presentation forms B*/
        U_FULL_WIDTH,    /* fullwidth ASCII*/
        U_HALF_WIDTH,    /* halfwidth CJK punctuation, Katakana, Hangul Jamo*/
        U_FULL_WIDTH,    /* fullwidth punctuation and currency signs*/
        U_HALF_WIDTH,    /* halfwidth forms, arrows, and shapes*/
        U_ZERO_WIDTH     /* specials*/
    };

const int16_t numCellWidthValues = 16;





/* Checks if ch is a lower case letter.*/
bool_t
u_islower(UChar ch) 
{
    /* Lazy evaluate the character type table; only instantiate when necessary.*/
    if (!tablesCreated) {
        createTables();
    }
    return (ucmp8_get(tables, ch) == U_LOWERCASE_LETTER);
}

/*Checks if ch is a upper case letter.*/
bool_t
u_isupper(UChar ch) 
{
    if (!tablesCreated) {
        createTables();
    }
    return (ucmp8_get(tables, ch) == U_UPPERCASE_LETTER);
}

/* Checks if ch is a title case letter; usually upper case letters.*/
bool_t
u_istitle(UChar ch) 
{
    if (!tablesCreated) {
        createTables();
    }
    return (ucmp8_get(tables, ch) == U_TITLECASE_LETTER);
}

/* Checks if ch is a decimal digit. */
bool_t
u_isdigit(UChar ch)
{    int8_t type;

    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return (type == U_DECIMAL_DIGIT_NUMBER || type == U_OTHER_NUMBER || type == U_LETTER_NUMBER);
}

/* Checks if ch is a unicode character with assigned character type.*/
bool_t
u_isdefined(UChar ch) 
{
    if (!tablesCreated) {
        createTables();
    }
    return (ucmp8_get(tables, ch) != U_UNASSIGNED);
}


/* Gets the character's linguistic directionality.*/
UCharDirection
u_charDirection(UChar ch )
{   
    /* Lazy evaluate the character directionality table; only instantiate when necessary.*/
    if (!dirTablesCreated) {
        createDirTables();
    }
    return (UCharDirection)(ucmp8_get(dirTables, ch));
}

/* Get the script associated with the character*/
UCharScript
u_charScript(UChar ch)
{
    int32_t index, j;
    UCharScript returnValue = U_NO_SCRIPT;

    index = -1;
    for( j = 0; index == -1 && fScriptIndex[j].fFirstCode != 0xFFFF; ++j )
        if( fScriptIndex[j].fFirstCode <= ch && ch <= fScriptIndex[j].fLastCode ) {
            index = j;
	    if(j == U_CHAR_SCRIPT_COUNT) /* "U_SPECIALS 2" */
	      index = U_SPECIALS;
        }
    if(index >= U_CHAR_SCRIPT_COUNT) {
        returnValue = U_NO_SCRIPT;
    }
    else if( index != -1 ) {
        returnValue = (UCharScript)index;
    } 

    return returnValue;
}

/* Checks if the Unicode character is a base form character that can take a diacritic.*/
bool_t
u_isbase(UChar ch)
{
    if (!tablesCreated) {
        createTables();
    }

    return (u_isalpha(ch) || u_isdigit(ch) ||
            (((((1 << U_NON_SPACING_MARK) |
                (1 << U_ENCLOSING_MARK) |
                (1 << U_COMBINING_SPACING_MARK)) >> u_charType(ch)) & 1) != 0));

}

/* Checks if the Unicode character is a control character.*/
bool_t
u_iscntrl(UChar ch)
{
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }

    type = ucmp8_get(tables, ch);
    return (type == U_CONTROL_CHAR ||
            type == U_FORMAT_CHAR ||
            type == U_LINE_SEPARATOR ||
            type == U_PARAGRAPH_SEPARATOR);
}

/* Checks if the Unicode character is printable.*/
bool_t
u_isprint(UChar ch)
{    
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return ((type >= U_UPPERCASE_LETTER && type <= U_PARAGRAPH_SEPARATOR) ||
            (type >= U_DASH_PUNCTUATION && type <= U_OTHER_SYMBOL));
}

/* Checks if the Unicode character is a letter.*/
bool_t
u_isalpha(UChar ch) 
{
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return ((isLetterMask & (1 << type)) != 0);
}


/* Checks if the Unicode character can start a Unicode identifier.*/
bool_t 
u_isIDStart(UChar ch)
{
    return(u_isalpha(ch));
}

/* Checks if the Unicode character can be a Unicode identifier part other than starting the
 identifier.*/
bool_t 
u_isIDPart(UChar ch)
{
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return(((isLetterMask & (1 << type)) != 0) ||
           (type == U_CONNECTOR_PUNCTUATION) ||
           (type == U_DECIMAL_DIGIT_NUMBER) ||
           (type == U_LETTER_NUMBER) ||
           (type == U_COMBINING_SPACING_MARK) || 
           (type == U_NON_SPACING_MARK) ||
           u_isIDIgnorable(ch));
}

/*Checks if the Unicode character can be ignorable in a Java or Unicode identifier.*/
bool_t 
u_isIDIgnorable(UChar ch)
{
    return( ( ch <= 0x0008) ||
            (ch >= 0x000E && ch <= 0x001B) ||
            (ch >= 0x007F && ch <= 0x009F) ||
            (ch >= 0x200A && ch <= 0x200F) ||
            (ch >= 0x206A && ch <= 0x206F) ||
            (ch == 0xFEFF) );
}
/*Checks if the Unicode character can start a Java identifier.*/
bool_t u_isJavaIDStart(UChar ch)
{
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return(((isLetterMask & (1 << type)) != 0) ||
           (type == U_CURRENCY_SYMBOL) ||
           (type == U_CONNECTOR_PUNCTUATION));
}

/*Checks if the Unicode character can be a Java identifier part other than starting the
 * identifier.
 */
bool_t u_isJavaIDPart(UChar ch)
{
    int8_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return(((isLetterMask & (1 << type)) != 0) ||
           (type == U_CURRENCY_SYMBOL) ||
           (type == U_CONNECTOR_PUNCTUATION) ||
           (type == U_DECIMAL_DIGIT_NUMBER) ||
           (type == U_LETTER_NUMBER) ||
           (type == U_COMBINING_SPACING_MARK) || 
           (type == U_NON_SPACING_MARK) ||
           u_isIDIgnorable(ch));
}
/* Transforms the Unicode character to its lower case equivalent.*/
UChar         
u_tolower(UChar ch) 
{
    UChar up_ch;

    if (!ulTablesCreated) {
        createUlTables();
    }
    
    /* special cases for title case characters*/
    if (u_istitle(ch)) 
    {
        switch(ch) 
        {
        case 0x01C5:
            return 0x01C4;
        case 0x01C8:
            return 0x01C7;
        case 0x01CB:
            return 0x01CA;
        case 0x01F2:
            return 0x01F1;
        default: return ch;
        }
    }

    else
    {
        if (((up_ch=(UChar)ucmp16_get(ulTables, ch))!= 0xFFFF)&&(u_isupper(ch))) return up_ch;
        else return ch;
    }
    
}
    
/*Transforms the Unicode character to its upper case equivalent.*/
UChar 
u_toupper(UChar ch) 
{
    UChar lw_ch; 
    if (!ulTablesCreated) {
        createUlTables();
    }
    
    /* return the upper case equivalent if the character is lower cased.*/
    if (((UChar)(lw_ch = ucmp16_get(ulTables, ch))!=0xFFFF)&&(u_islower(ch))) return lw_ch;
    else return ch;
}

/* Transforms the Unicode character to its title case equivalent.*/
UChar 
u_totitle(UChar ch)
{
    switch(ch) {
    case 0x01C4: case 0x01C5: case 0x01C6:
        return 0x01C5;
    case 0x01c7: case 0x01C8: case 0x01C9:
        return 0x01C8;
    case 0x01ca: case 0x01CB: case 0x01CC:
        return 0x01CB;
    case 0x01f1: case 0x01F2: case 0x01F3:
        return 0x01F2;
    default:
        return u_toupper(ch);
    }
}

/* Checks if the Unicode character is a space character.*/
bool_t
u_isspace(UChar ch) 
{
    int32_t type;
    if (!tablesCreated) {
        createTables();
    }
    type = ucmp8_get(tables, ch);
    return ((type == U_SPACE_SEPARATOR) ||
            (type == U_LINE_SEPARATOR) ||
            (type == U_PARAGRAPH_SEPARATOR));
}

/* Gets if the Unicode character's character property.*/
int8_t
u_charType(UChar ch)
{
    if (!tablesCreated) {
        createTables();
    }
    return ucmp8_get(tables, ch);
}



/* Gets table cell width of the Unicode character.*/
uint16_t
u_charCellWidth(UChar ch)
{
    int16_t i;
    int32_t type = u_charType(ch);

    /* these Unicode character types are scattered throughout the Unicode range, so
     special-case for them*/
    switch (type) {
        case U_UNASSIGNED:
        case U_NON_SPACING_MARK:
        case U_ENCLOSING_MARK:
        case U_LINE_SEPARATOR:
        case U_PARAGRAPH_SEPARATOR:
        case U_CONTROL_CHAR:
        case U_FORMAT_CHAR:
            return U_ZERO_WIDTH;

        default:
            /* for all remaining characters, find out which Unicode range they belong to using
               the table above, and then look up the appropriate return value in that table*/
            for (i = 0; i < numCellWidthValues; ++i)
                if (ch < cellWidthRanges[i])
                    break;
            --i;
            return cellWidthValues[i];
    }
}

int32_t            
u_charDigitValue(UChar ch)
{
    int32_t returnValue = -1;
    int32_t i = 0;
    while (u_isdefined(fCodeDigitTable[i].fUnicode)) {
        if ( ch == fCodeDigitTable[i].fUnicode ) {
            returnValue = fCodeDigitTable[i].fValue;
            break;
        }
        i++;
    }    
    return returnValue;
}

void
createTables()
{
    CompactByteArray* newTables;
    if (tables == 0) {
      newTables = ucmp8_openAdopt((uint16_t*)indicies, (int8_t*)values, offsetCount);    
      umtx_lock(NULL);
      if (tables != 0) {
    ucmp8_close(newTables);
      }
      else {
    tables = newTables;
      }
    }

#ifndef TESTING_CODE_COVERAGE
    if (tables) tablesCreated = TRUE;
#endif
    umtx_unlock(NULL);
}

void
createUlTables()
{
  CompactShortArray* newTables;
  if (ulTables == 0) 
    {
      newTables = ucmp16_openAdopt((uint16_t*)caseIndex,
                   (int16_t*)caseValues,
                   caseCount,
                   0);
      umtx_lock(NULL);

      if (ulTables != 0)
    {
      ucmp16_close(newTables);
    }
      else 
    {
      ulTables = newTables;
        }
    }
#ifndef TESTING_CODE_COVERAGE
    if (ulTables) ulTablesCreated = TRUE;
#endif
    umtx_unlock(NULL);
}


void
createDirTables()
{
  CompactByteArray* newTables;
  if (dirTables == 0)
    {
      newTables = ucmp8_openAdopt((uint16_t*)fCharDirIndices,
                  (int8_t*)fCharDirValues,
                  fCharDirCount);
      umtx_lock(NULL);
      if (dirTables != 0) 
    {
      ucmp8_close(newTables);
    }
      else {
    dirTables = newTables;
      }
    }
#ifndef TESTING_CODE_COVERAGE
  if (dirTables) dirTablesCreated = TRUE;
#endif
  umtx_unlock(NULL);
}

const char* u_getVersion()
{
	int32_t len=strlen(UNICODE_VERSION) + strlen("Unicode Version ");
	_ucdVersion=(char*)icu_realloc(_ucdVersion, len + 1 );
	strcpy(_ucdVersion, "Unicode Version ");
	strcat(_ucdVersion, UNICODE_VERSION);
	return _ucdVersion;
}
	

